/*
 * Copyright (C) 2005 - 2006
 * Anastasios Skarlatidis
 *
 * This program developed as part of the dissertation entitled:
 * A Framework for Implementation and Visualization of Evolutionary Algorithms,
 * by Anastasios Skarlatidis.
 * Technological Educational Institute of Thessaloniki,
 * Department of Information Technology
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 */

package examples.wasps;

import gr.teithe.it.util.annotations.Require;
import gr.teithe.it.util.annotations.GUIConfiguration;
import gr.teithe.it.ga.representation.IntegerRepresentation;
import gr.teithe.it.ga.representation.genes.IGene;
import gr.teithe.it.ga.representation.genes.IntegerGene;
import gr.teithe.it.ga.eval.AbstractMinimizationEvaluator;
import gr.teithe.it.ga.IChromosome;
import gr.teithe.it.ga.configuration.IGUIConfigurator;
import javolution.util.FastList;
import javolution.realtime.LocalContext;
import javolution.xml.XmlFormat;
import javolution.xml.XmlElement;

import javax.swing.*;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;

/**
 * @author <a href="mailto:anskarl@yahoo.com">Anastasios Skarlatidis</a>
 */
@Require( value = IntegerRepresentation.class)
@GUIConfiguration(value = WaspBombsIntEvaluator.GUIConf.class)
public final class WaspBombsIntEvaluator extends AbstractMinimizationEvaluator {

    private static final String ATTR_DATA_FILE = "DataFile";

    public static final XmlFormat<WaspBombsIntEvaluator> XML = new XmlFormat<WaspBombsIntEvaluator>(WaspBombsIntEvaluator.class) {
        public void format(WaspBombsIntEvaluator waspBombsIntEvaluator, XmlElement xmlElement) {
            xmlElement.setAttribute(ATTR_DATA_FILE,waspBombsIntEvaluator.file.getAbsolutePath());
        }

        public WaspBombsIntEvaluator parse(XmlElement xmlElement) {
            return new WaspBombsIntEvaluator(new File(xmlElement.getAttribute(ATTR_DATA_FILE, "nests.dat")));
        }
    };

    private FastList<WaspsNest> list=null;
    private WaspNestsLoader loader=null;
    private File file = null;
    private static final double dmax = 141.42;


    public WaspBombsIntEvaluator(File file){
        super();

        if(file==null)
            this.file = new File("nests.dat");
        else
            this.file = file;

        try {
            loader = new WaspNestsLoader(file);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        try {
            list = loader.load();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    public void evaluate(IChromosome c) {
        IGene[] genes = c.getGenes();
        try{
            LocalContext.enter();
            FastList<WaspsNest> currList= FastList.newInstance();
            for (FastList.Node<WaspsNest> n = list.head(), end = list.tail(); (n = n.getNext()) != end;)
                currList.addLast(n.getValue().copy());

            double fitness = 0.0;
            double k;

            for(int i=0; i<genes.length; i+=2){
                k = 0.0;
                int bombX = ((IntegerGene) genes[i]).getValue();
                int bombY = ((IntegerGene) genes[i+1]).getValue();
                for(WaspsNest node: currList){
                    k = node.numberOfWasps*(WaspBombsIntEvaluator.dmax /
                            ((20*distance(bombX, bombY, node.x, node.y))+0.00001));
                    if(node.numberOfWasps!=0){
                        if(k>node.numberOfWasps) node.numberOfWasps=0;
                        else node.numberOfWasps-=k;
                    }
                }
            }

            for(WaspsNest node: currList){
                fitness+=node.numberOfWasps;
            }
            c.setFitnessValue(fitness);
        }finally{
            LocalContext.exit();
        }
    }

    private final double distance(int x1, int y1, int x2, int y2){
        return Math.sqrt(Math.pow(x1-x2,2)+Math.pow(y1-y2, 2));
    }

    public final static class GUIConf extends JPanel implements IGUIConfigurator<WaspBombsIntEvaluator> {

        private final JButton btnOpenFile = new JButton("Open File");
        private final JFileChooser fileChooser = new JFileChooser();
        private File file = null;

        public GUIConf(){
            fileChooser.setDialogTitle("Open wasp nests file");
            btnOpenFile.addActionListener(new ActionListener() {
                public void actionPerformed(ActionEvent e) {
                    if(fileChooser.showOpenDialog(null)==JFileChooser.APPROVE_OPTION){
                        file = fileChooser.getSelectedFile();
                    }
                }
            });
            add(btnOpenFile);
        }

        public JComponent getJComponent() {
            return this;
        }

        public WaspBombsIntEvaluator getConfiguredType() {
            return new WaspBombsIntEvaluator(file);
        }
    }

}
